;===========================================================================================
; Date.asm  Gosm assembler
; Afficher le jour / mois / anne
; Programm par AsmGges France
;===========================================================================================
; compile.bat:
; ..\..\bin\GoRC /r, res\date.rc
; ..\..\bin\GoAsm date.asm
; ..\..\bin\GoLink date.obj res\date.res user32.dll kernel32.dll gdi32.dll
;===========================================================================================

; Inclusion des macros/quates/structures ncessaires au programme
; ----------------------------------------------------------------
  #include "..\..\inc\Windows.inc"
  #include "..\..\inc\all_api.inc"

  ID_FRANCE = 17

  WSCREEN   = 230
  HSCREEN   = 62

; Macro for wsprintf access
cinvoke(funcname,%1,%2,%3,%4,%5) = \ 
                               #if ARGCOUNT=1                     \ 
                                   invoke funcname                \ 
                               #elif ARGCOUNT=2                   \ 
                                   invoke funcname,%1             \ 
                               #elif ARGCOUNT=3                   \ 
                                   invoke funcname,%1,%2          \ 
                               #elif ARGCOUNT=4                   \ 
                                   invoke funcname,%1,%2,%3       \ 
                               #elif ARGCOUNT=5                   \ 
                                   invoke funcname,%1,%2,%3,%4    \ 
                               #elif ARGCOUNT=6                   \ 
                                   invoke funcname,%1,%2,%3,%4,%5 \ 
                               #endif                             \
                               #if ARGCOUNT>1                     \ 
                                   add esp,ARGCOUNT-1*4           \
                               #endif

;===========================================================================================
; Dbut du code programme
;-------------------------------------------------------------------------------------------
.code

start:
    ; Obtenir le handle du programme
    ; ------------------------------
    invoke GetModuleHandle,0
    mov [wc.hInstance],eax

    ; Chargement d'une icne
    ; ----------------------
    invoke LoadIcon,[wc.hInstance],ID_FRANCE
    mov [wc.hIcon],eax

    ; Chargement d'un curseur
    ; -----------------------
    invoke LoadCursor,0,IDC_ARROW
    mov [wc.hCursor],eax

    ; Valider le choix de la structure WNDCLASS
    ; -----------------------------------------
    invoke RegisterClass,addr wc

    ; Obtenir la largeur en pixels de l'cran
    ; ---------------------------------------
    invoke GetSystemMetrics,SM_CXSCREEN
    sub eax,WSCREEN
    shr eax,1
    mov [rect.left],eax

    ; Ajuster les coordonnes de la fentre
    ; -------------------------------------
    mov D[rect.top],0
    mov D[rect.right],WSCREEN
    mov D[rect.bottom],HSCREEN
    invoke AdjustWindowRectEx,addr rect,WS_CAPTION+WS_VISIBLE+WS_SYSMENU+WS_MINIMIZEBOX,\
                                                                                 0,NULL
    mov eax,[rect.bottom]
    sub eax,3                ;hauteur bord infrieur fentre active
    sub eax,[rect.top]
    sub eax,HSCREEN
    inc eax                  ;hauteur bord infrieur bandeau titre
    mov [HBarTitle],eax      ;hauteur totale du bandeau titre

    invoke GetSystemMetrics,SM_CYSCREEN
    sub eax,HSCREEN
    sub eax,[HBarTitle]
    shr eax,1
    mov D[rect.top],100      ;forage de Y sinon remplacer la valeur par eax

    mov eax,HSCREEN
    add eax,[HBarTitle]

    ; Cration fentre principale avec barre de titre suprieure
    ; ----------------------------------------------------------
    invoke CreateWindowEx,NULL,addr ClassName,addr WinTitreFr,\
                          WS_CAPTION+WS_VISIBLE+WS_SYSMENU+WS_MINIMIZEBOX,\
                          [rect.left],[rect.top],WSCREEN+6,eax,NULL,0,[wc.hInstance],NULL
    mov [hWin],eax

    ; Affichage de la fentre
    ; -----------------------
    invoke ShowWindow,[hWin],SW_SHOWDEFAULT
    ; Rafraichissement de la fentre
    ; ------------------------------
    invoke UpdateWindow,[hWin]

 ; Boucle de traitement des messages
 ; ---------------------------------
 MessageLoop:
    invoke GetMessage,addr msg,0,0,0
    or eax,eax
    jz >ExitProgram
    invoke TranslateMessage,addr msg
    invoke DispatchMessage,addr msg
    jmp <MessageLoop

 ExitProgram:
    ; Fermeture de l'application
    ; --------------------------
    invoke ExitProcess,[msg.wParam]

;===========================================================================================
; Procdure principale Windows
;-------------------------------------------------------------------------------------------
WindowProc frame @hWnd, @uMsg, @wParam, @lParam
    uses ebx,esi,edi

    cmp D[@uMsg],WM_CLOSE
    jne >.wmcreate
        ; Dtruire le timer
        ; -----------------
        invoke KillTimer,[@hWnd],1000

        ; Boite  message fin d'application
        ; ---------------------------------
        invoke MessageBox,[@hWnd],addr MsgPub,addr ClassName,MB_OK+MB_ICONINFORMATION

        ; Demande la fin de l'application
        ; -------------------------------
        invoke PostQuitMessage,0
        jmp >>.exit

    .wmcreate:
    cmp D[@uMsg],WM_CREATE
    jne >>.wmtimer
        ; Cration fonte
        ; --------------
        invoke CreateFont,8,0,0,0,400,0,0,0,ANSI_CHARSET,OUT_DEFAULT_PRECIS,\
                          CLIP_DEFAULT_PRECIS,DEFAULT_QUALITY,DEFAULT_PITCH+FF_SWISS,\
                                                                         addr FontMS
        mov [hFontMS],eax

        ; Groupbox AsmGges
        ; ----------------
        invoke Ctl,addr Button,addr ClassName,[@hWnd],8,2,WSCREEN-16,20,-1,BS_GROUPBOX,0
        ; Affecter la fonte au control
        ; ----------------------------
        invoke SendMessage,eax,WM_SETFONT,[hFontMS],NULL

        ; Cration d'une fentre static pour l'affichage de la date
        ; ---------------------------------------------------------
        invoke Ctl,addr Static,addr NoTxt,[@hWnd],8,32,WSCREEN-16,22,-1,SS_CENTER,\
                                                                 WS_EX_CLIENTEDGE
        mov [hStatic],eax
        ; Affecter la fonte au control
        ; ----------------------------
        invoke SendMessage,[hStatic],WM_SETFONT,[hFontMS],NULL

        ; Initialiser un timer
        ; --------------------
        invoke SetTimer,[@hWnd],1000,100,NULL
        jmp >>.exit

    .wmtimer:
    cmp D[@uMsg],WM_TIMER
    jne >.wmpaint
        ; Jour mois anne
        ; ---------------
        invoke GetLocalTime,addr stt
        mov eax,0

        mov ax,[stt.wDay]
        mov [mDay],eax

        mov ax,[stt.wMonth]
        mov [mMonth],eax

        mov ax,[stt.wYear]
        mov [mYear],eax

        ; Concat des chanes
        ; ------------------
        cinvoke(wsprintf,addr lpTodayDMY,addr lpFmtDate,[mDay],[mMonth],[mYear])

        ; Texte complet dans la fentre statique
        ; --------------------------------------
        invoke SetWindowText,[hStatic],addr lpTodayDMY
        jmp >.exit

    .wmpaint:
    cmp D[@uMsg],WM_PAINT
    jne >.defwindowproc
        ; Dbut criture/dessin
        ; ---------------------
        invoke BeginPaint,[@hWnd],addr ps

        ; Fin criture/dessin
        ; -------------------
        invoke EndPaint,[@hWnd],addr ps
        jmp >.exit

    .defwindowproc:
        ; Traitement par dfaut des autres messages Windows
        ; -------------------------------------------------
        invoke DefWindowProc,[@hWnd],[@uMsg],[@wParam],[@lParam]
        ret

    .exit:

    mov eax,0
    ret

endf

;===========================================================================================
; Cration d'une fentre-enfant control class
;-------------------------------------------------------------------------------------------
; @Class   = adresse du nom de control class
; @AdrText = adresse du control text
; @hParent = handle de la fentre parent
; @X       = x-coordonne fentre-enfant
; @Y       = y-coordonne fentre-enfant
; @Xw      = largeur en pixels fentre-enfant
; @Yh      = hauteur en pixels fentre-enfant
; @Id      = identificateur du control forme ID_XXX ou -1 si pas de ID
; @Style   = complment window style de WS_CHILD+WS_VISIBLE
; @ExStyle = window style tendu
;-------------------------------------------------------------------------------------------
; => eax = handle de la fentre-enfant
;-------------------------------------------------------------------------------------------
Ctl frame @Class, @AdrText, @hParent, @X, @Y, @Xw, @Yh, @Id, @Style, @ExStyle

    mov eax,WS_CHILD+WS_VISIBLE
    add eax,[@Style]
    invoke CreateWindowEx,[@ExStyle],[@Class],[@AdrText],eax,[@X],[@Y],[@Xw],[@Yh],\
                          [@hParent],[@Id],[wc.hInstance],0
    ret
endf

;===========================================================================================
; Dfinitions des datas chaines / variables initialises ou non initialises / structures
;-------------------------------------------------------------------------------------------
.data

  ClassName    db ' AsmGges Win32',0
  WinTitreFr   db ' Date et Windows  ',0
  MsgPub       db ' GoAsm AsmGges France @2006  ',0

  Static       db 'STATIC',0
  Button       db 'BUTTON',0
  FontMS       db 'Ms Sans Serif',0
  NoTxt        db '',0
  lpFmtDate    db '%02d/%02d/%04d',0
  align 4

  mDay         dd 0
  mMonth       dd 0
  mYear        dd 0

  hWin         dd ?
  hFontMS      dd ?
  HBarTitle    dd ?
  hStatic      dd ?

  lpTodayDMY   db 11 dup 0
  align 4

  wc           WNDCLASS    < 0, addr WindowProc, 0, 0, 0, 0, 0, \
                             COLOR_BTNFACE+1, 0, addr ClassName >
  msg          MSG         <>
  ps           PAINTSTRUCT <>
  rect         RECT        <>
  stt          SYSTEMTIME  <>

;===========================================================================================
